#!/bin/sh
#
#
# Absolutely NO warranty.
# This script will overwrite one of your Volumes
#
# You must run this script as the root user
#    sudo -s
#
# OR
#
# sudo buildInstallerDrive.sh [path]
#
# It should work for both v2.x and v3.x boot.efi files from Pike R Alpha
# Place a copy of your choice of efi file into one of the following locations
#        The same folder as this script $scriptDir
#        /Extras
#        /opt
#        /Library/Application Support/Boot64
#
# Otherwise use and abuse as you like
# 
#
# CHANGE THIS - some common values
#					/Volumes/Untitled
#					/Volumes/Untitled\ 1
###########################################
TargetDiskToBeWipedBeforeCreatingInstaller="/Volumes/Install OS X El Capitan"
###########################################
#
# Or pass the path to the script as its argument
# E.G.
# buildInstallerDrive.sh /Volumes/Untitled
#
# Inspector42 on 2015-11-08:
#
# Modified to follow Peter Hoolbrooks guide http://forums.macrumors.com/threads/boot-efi-developers-thread.1924434/page-32#post-22213699
# Section to add Board-IDs to enable installing on older Macs currently commented out as it is not needed with version 3.x
#
# Needs pbzx installed either in same path as this script of in standard program paths, e.g. /usr/local/bin
# replaced cp with rsync for the big copy chunks to enable progress indication
# Exit Codes in accordance with which step failed:
# 0: Everything went as intended
# 1: Not run as super user
# 2: Missing appropriate Volume to copy to
# 3: Confirmation of erasing target volume was not answered with "y"
# 4: Installer-App for El Capitan missing in Applications folder
# 5: pbzx-tool missing
# 6: boot.efi missing
# 7: BaseSystem Image could not be restored
#
#

effectiveUser=`id -u`
if [ $effectiveUser -ne 0 ]; then
	echo ""
	echo "############################################################"
	echo ""
	echo "This script must be run as the root user"
	echo " try:"
	echo "	sudo -s"
	echo "	Then re-run the script"
	echo ""
	echo " or:"
	echo ""
	echo "	sudo '$0'"
	echo ""
	echo "############################################################"
	echo ""
	exit -1
fi

if [ "Q$*" = "Q" ]; then
	if [ -d "$TargetDiskToBeWipedBeforeCreatingInstaller" ]; then
		echo "# ! Using default volume '$TargetDiskToBeWipedBeforeCreatingInstaller'"
	else
		echo ""
		echo " Please supply a volume to use"
		echo " Or name a volume '$TargetDiskToBeWipedBeforeCreatingInstaller'"
		echo ""
		echo ""
		exit -2
	fi
else
	TargetDiskToBeWipedBeforeCreatingInstaller="$*"
	echo "############################################################"
	echo ""
	echo " WARNING"
	echo " WARNING"
	echo " WARNING"
	echo " WARNING"
	echo " WARNING"
	echo ""
	echo " The volume '$TargetDiskToBeWipedBeforeCreatingInstaller' will be ERASED"
	echo ""
	echo "############################################################"
	echo ""
	read -p "Okay to ERASE '$TargetDiskToBeWipedBeforeCreatingInstaller'? " yn
	if [ "$yn" != "y" ]; then
		exit -3
	fi
fi

if [ ! -d "/Applications/Install OS X El Capitan.app" ]; then
	echo ""
	echo "######################################################################################################"
	echo "#"
	echo "# The El Capitan Installer App ('Install OS X El Capitan.app') needs to be in the Applications folder"
	echo "#"
	echo "######################################################################################################"
	echo ""
	exit -4
fi

if [  ! -f pbzx ]; then
    pbzxPath=`which pbzx`
    if [ $? = 1 ]; then
	    echo ""
	    echo "########################################################################################"
	    echo "#"
        echo "# Could not find the pbzx tool"
	    echo "#    Download from http://www.tonymacx86.com/general-help/135458-pbzx-stream-parser.html"
	    echo "#        Registration required"
	    echo "#"
	    echo "########################################################################################"
	    echo ""
	    exit -5
    fi
else
scriptDir="$( cd "$(dirname "$0")" ; pwd -P )"
pbzxPath="$scriptDir/pbzx"
fi

echo "$pbzxPath"

scriptDir="$( cd "$(dirname "$0")" ; pwd -P )"
efiPath="$scriptDir/boot.efi"

if [ ! -f "$efiPath" ]; then
	if [ ! -f "/Extras/boot.efi" ]; then
		if [ ! -f "/opt/boot.efi" ]; then
			if [ ! -f "/Library/Application Support/Boot64/boot.efi" ]; then
				echo ""
				echo "########################################################################################"
				echo "#"
				echo "# Could not find a boot.efi file"
				echo "#    Place a Pike R Alpha boot.efi file into one of the following locations:"
				echo "#        The same folder as this script $scriptDir"
				echo "#        /Extras"
				echo "#        /opt"
				echo "#        /Library/Application Support/Boot64"
				echo "#"
				echo "########################################################################################"
				echo ""
				exit -6
			else
				efiPath="/Library/Application Support/Boot64/boot.efi"
			fi
		else
			efiPath="/opt/boot.efi"
		fi
	else
		efiPath="/Extras/boot.efi"
	fi
fi

echo "$efiPath"

	
#
# Turn off spotlight indexing to ensure we can unmount the volume
mdutil -i off "$TargetDiskToBeWipedBeforeCreatingInstaller"
#
# It is assumed that your copy of the El Capitan Installer app is in /Applications
#
echo "\n\nAttach the InstallESD disk image from the Installer bundle"
hdiutil attach /Applications/Install\ OS\ X\ El\ Capitan.app/Contents/SharedSupport/InstallESD.dmg

echo "\n\nRestore the Base System from the InstallESD"
asr restore --source /Volumes/OS\ X\ Install\ ESD/BaseSystem.dmg --target "$TargetDiskToBeWipedBeforeCreatingInstaller" --erase --noprompt
if [ $? -eq 1 ]; then
	echo ""
	echo "########################################################################################"
	echo "#"
	echo "# Could not restore the InstallESD image to $TargetDiskToBeWipedBeforeCreatingInstaller"
	echo "#"
	echo "########################################################################################"
	echo ""
	exit -7
fi


echo "\n\nRename the installer volume to 'Install OS X El Capitan'"
diskutil rename /Volumes/OS\ X\ Base\ System/ "Install OS X El Capitan"
#
# We have just renamed the drive - so now use the known name
TargetDiskToBeWipedBeforeCreatingInstaller="/Volumes/Install OS X El Capitan"

echo "\n\nRemove the Packages link"
rm "$TargetDiskToBeWipedBeforeCreatingInstaller/System/Installation/Packages"

echo "\n\nCopy the full Packages directory from the InstallESD to the new installer volume"
# cp -R /Volumes/OS\ X\ Install\ ESD/Packages/ "$TargetDiskToBeWipedBeforeCreatingInstaller/System/Installation/Packages"
# using rsync to have some progress indication during this lengthy copy operation, in particular on slow USB sticks
rsync -ah --progress /Volumes/OS\ X\ Install\ ESD/Packages/ "$TargetDiskToBeWipedBeforeCreatingInstaller/System/Installation/Packages"

echo "\n\nCopy the BaseSystem disk images to the root of the new Installer volume..."
# cp /Volumes/OS\ X\ Install\ ESD/BaseSystem* "$TargetDiskToBeWipedBeforeCreatingInstaller/"
rsync -ah --progress /Volumes/OS\ X\ Install\ ESD/BaseSystem* "$TargetDiskToBeWipedBeforeCreatingInstaller/"

echo "\n\nReplace files:\n    Unlock  System/Library/CoreServices/boot.efi"
chflags nouchg "$TargetDiskToBeWipedBeforeCreatingInstaller/System/Library/CoreServices/boot.efi"
echo "    Replace System/Library/CoreServices/boot.efi"
cp "$efiPath" "$TargetDiskToBeWipedBeforeCreatingInstaller/System/Library/CoreServices/boot.efi"
chmod 644 "$TargetDiskToBeWipedBeforeCreatingInstaller/System/Library/CoreServices/boot.efi"
chown root:wheel "$TargetDiskToBeWipedBeforeCreatingInstaller/System/Library/CoreServices/boot.efi"
echo "\n    Lock  System/Library/CoreServices/boot.efi"
chflags uchg "$TargetDiskToBeWipedBeforeCreatingInstaller/System/Library/CoreServices/boot.efi"
echo "    Replace System/Library/CoreServices/bootbase.efi"
cp "$efiPath" "$TargetDiskToBeWipedBeforeCreatingInstaller/System/Library/CoreServices/bootbase.efi"
chmod 644 "$TargetDiskToBeWipedBeforeCreatingInstaller/System/Library/CoreServices/bootbase.efi"
chown root:wheel "$TargetDiskToBeWipedBeforeCreatingInstaller/System/Library/CoreServices/bootbase.efi"
echo "    Replace usr/standalone/i386/boot.efi"
cp "$efiPath" "$TargetDiskToBeWipedBeforeCreatingInstaller/usr/standalone/i386/boot.efi"
chmod 644 "$TargetDiskToBeWipedBeforeCreatingInstaller/usr/standalone/i386/boot.efi"
chown root:wheel "$TargetDiskToBeWipedBeforeCreatingInstaller/usr/standalone/i386/boot.efi"

#
# Inspector42: 2015-11-08
# This section is not needed if using version 3.1 of Pike´s boot.efi
#
#echo "    Replace System/Library/CoreServices/PlatformSupport.plist"
#awk '{ if ( $1 ~ /<array>/ ) { print; print "\t\t<string>Mac-F4208DC8</string>"; print "\t\t<string>Mac-F4208DA9</string>" } else { print } }' "$TargetDiskToBeWipedBeforeCreatingInstaller/System/Library/CoreServices/PlatformSupport.plist" > /tmp/PlatformSupport.$$.plist
#cp /tmp/PlatformSupport.$$.plist "$TargetDiskToBeWipedBeforeCreatingInstaller/System/Library/CoreServices/PlatformSupport.plist"
#echo "    Replace System/Library/CoreServices/com.apple.recovery.boot/PlatformSupport.plist"
#cp /tmp/PlatformSupport.$$.plist "$TargetDiskToBeWipedBeforeCreatingInstaller/System/Library/CoreServices/com.apple.recovery.boot/PlatformSupport.plist"
#echo "    Replace System/Installation/Packages/InstallableMachines.plist"
#cp /tmp/PlatformSupport.$$.plist "$TargetDiskToBeWipedBeforeCreatingInstaller/System/Installation/Packages/InstallableMachines.plist"

#echo "\n\nModify System/Installation/Packages/OSInstall.mpkg"
#pkgutil --expand /Volumes/OS\ X\ Install\ ESD/Packages/OSInstall.mpkg /tmp/OSInstall-unpacked.$$
#mv /tmp/OSInstall-unpacked.$$/Distribution /tmp/OSInstall-unpacked.$$/Distribution.apple
#sed 's/var platformSupportValues=\[/var platformSupportValues=["Mac-F4208DC8","Mac-F4208DA9",/' /tmp/OSInstall-unpacked.$$/Distribution.apple > /tmp/OSInstall-unpacked.$$/Distribution
#rm /tmp/OSInstall-unpacked.$$/Distribution.apple
#cd /tmp
#mv "$TargetDiskToBeWipedBeforeCreatingInstaller/System/Installation/Packages/OSInstall.mpkg" "$TargetDiskToBeWipedBeforeCreatingInstaller/System/Installation/Packages/OSInstall.mpkg.apple"
#pkgutil --flatten /tmp/OSInstall-unpacked.$$/ "$TargetDiskToBeWipedBeforeCreatingInstaller/System/Installation/Packages/OSInstall.mpkg"
#
#
# So now we need to actually unpack the Payload file, this should be a LZVN compressed CPIO file
# The Hackintosh community built a handy utility to extract the Payload, pbzx
#    http://www.tonymacx86.com/general-help/135458-pbzx-stream-parser.html
echo ""
echo "\n\nDecompress Essentials.pkg to extract the kernel. This will take some time..."
cd "$TargetDiskToBeWipedBeforeCreatingInstaller"
"$pbzxPath" /Volumes/OS\ X\ Install\ ESD/Packages/Essentials.pkg | cpio -idmu ./System/Library/Kernels/kernel

echo ""
echo "\n\nBlessing the Installer"
bless --folder "$TargetDiskToBeWipedBeforeCreatingInstaller/System/Library/CoreServices" --file "$TargetDiskToBeWipedBeforeCreatingInstaller/System/Library/CoreServices/boot.efi"
echo ""

#if [ ! -d "$TargetDiskToBeWipedBeforeCreatingInstaller/System/Installation/Packages/Extras" ]; then
#	mkdir -p "$TargetDiskToBeWipedBeforeCreatingInstaller/System/Installation/Packages/Extras"
#fi

#echo " -allowUntrusted " > "$TargetDiskToBeWipedBeforeCreatingInstaller/System/Installation/Packages/Extras/AdditionalInstallerArgs"

echo "\n\nDone."
echo ""
hdiutil unmount "/Volumes/OS X Install ESD"
exit 0
